import os
import sys
import struct

ARGV = sys.argv

RM = 'rm -f '
VM = './vm '
TINYPY = './tinypy '
if '-mingw32' in ARGV:
    RM = 'del '
    VM = 'vm '
    TINYPY = 'tinypy '

def number(v):
    if type(v) is str and v[0:2] == '0x':
        v = int(v[2:],16)
    return float(v)

def istype(v,t):
    if t == 'string': return isinstance(v,str)
    elif t == 'list': return (isinstance(v,list) or isinstance(v,tuple))
    elif t == 'dict': return isinstance(v,dict)
    elif t == 'number': return (isinstance(v,float) or isinstance(v,int))
    raise '?'

def fpack(v):
    return struct.pack('f',v)

def system(cmd):
    return os.system(cmd)

def load(fname):
    f = open(fname,'rb')
    r = f.read()
    f.close()
    return r

def save(fname,v):
    f = open(fname,'wb')
    f.write(v)
    f.close()

def do_cmd(cmd):
    FLAGS = '`sdl-config --cflags --libs`'
    SYS = '-linux'
    if '-mingw32' in sys.argv:
        #-mwindows -mno-cygwin 
        FLAGS = '-mwindows -lmingw32 -lSDLmain -Ic:/packages/sdl-devel/include -Lc:/packages/sdl-devel/lib -lSDL -Lc:/packages/sdl-devel/lib -lSDL'
        SYS = '-mingw32'
    cmd = cmd.replace('$FLAGS',FLAGS)
    cmd = cmd.replace('$SYS',SYS)
    print cmd
    r = os.system(cmd)
    if r:
        print 'exit',r
        sys.exit(r)

def chksize():
    import mk64k
    t1,t2 = 0,0
    for fname in [
        'pylang.py','dumbparse.py','dump2vm.py','py2c.py',
        'tp.h','tp.c','dict.c','list.c','vm.c',
        'string.c','builtins.c','util.c','tgc.c',
        'tinypy.c',
        ]:
        f = open(fname,'r'); t1 += len(f.read()); f.close()
        txt = mk64k.shrink(fname)
        t2 += len(txt)
    print "# ",t1,t2
    return t2


def build_bc(opt=False):
    out = []
    mods = ['pylang','dumbparse','dump2vm','py2c']
    for mod in mods:
        out.append("""unsigned char tp_%s[] = {"""%mod)
        fname = mod+".tpc"
        data = open(fname,'rb').read()
        cols = 16
        for n in xrange(0,len(data),cols):
            out.append(",".join([str(ord(v)) for v in data[n:n+cols]])+',')
        out.append("""};""")
    out.append("")
    f = open('tpbc.c','wb')
    f.write('\n'.join(out))
    f.close()
    #do_cmd("rm -f ./tests.tpc "+" ".join(["./"+mod+".tpc" for mod in mods]))

def bootstrap():
    do_cmd("gcc -Wall -g vmmain.c $FLAGS -lm -o vm")
    do_cmd('python tests.py $SYS')
    do_cmd('python py2c.py tests.py tests.tpc')
    do_cmd('python py2c.py pylang.py pylang.tpc')
    do_cmd('python py2c.py dumbparse.py dumbparse.tpc')
    do_cmd('python py2c.py dump2vm.py dump2vm.tpc')
    do_cmd('python py2c.py py2c.py py2c.tpc')
    do_cmd(VM+'tests.tpc $SYS')
    do_cmd(VM+'py2c.tpc tests.py tests.tpc')
    do_cmd(VM+'py2c.tpc pylang.py pylang.tpc')
    do_cmd(VM+'py2c.tpc dumbparse.py dumbparse.tpc')
    do_cmd(VM+'py2c.tpc dump2vm.py dump2vm.tpc')
    do_cmd(VM+'py2c.tpc py2c.py py2c.tpc')
    build_bc()
    do_cmd("gcc -Wall -g tinypy.c $FLAGS -lm -o tinypy")
    #second pass - builts optimized binaries and stuff
    do_cmd(TINYPY+'tests.py $SYS')
    do_cmd(TINYPY+'py2c.py tests.py tests.tpc -nopos')
    do_cmd(TINYPY+'py2c.py pylang.py pylang.tpc -nopos')
    do_cmd(TINYPY+'py2c.py dumbparse.py dumbparse.tpc -nopos')
    do_cmd(TINYPY+'py2c.py dump2vm.py dump2vm.tpc -nopos')
    do_cmd(TINYPY+'py2c.py py2c.py py2c.tpc -nopos')
    build_bc(True)
    do_cmd("gcc -Wall -O2 tinypy.c $FLAGS -lm -o tinypy")
    do_cmd(TINYPY+'tests.py $SYS')
    print("OK")

if __name__ == '__main__':
    bootstrap()
