int tp_lua_hash(void *v,int l) {
    unsigned char *s = v;
    int i,h=l,step = (l>>5)+1;
    for (i=l; i>=step; i-=step) {
        h = h^((h<<5)+(h>>2)+s[i-1]);
    }
    return h;
}
void _tp_dict_free(_tp_dict *self) {
    tp_free(self->items);
    tp_free(self);
}

void _tp_dict_reset(_tp_dict *self) {
    memset(self->items,0,self->alloc*sizeof(tp_item));
    self->len = 0;
    self->used = 0;
    self->cur = 0;
}

int tp_hash(TP,tp_obj v) {
    int type = v.type;
    if (type == TP_NONE) {
        return 0;
    } else if (type == TP_NUMBER) {
        return tp_lua_hash(&v.number.val,sizeof(float));
    } else if (type == TP_STRING) {
        return tp_lua_hash(v.string.val,v.string.len);
    } else if (type == TP_DICT) {
        return tp_lua_hash(&v.dict.val,sizeof(void*));
    } else if (type == TP_LIST) {
        return tp_lua_hash(&v.list.val,sizeof(void*));
    } else if (type == TP_DEF) {
        return tp_lua_hash(&v.def.val,3*sizeof(void*));
    } else if (type == TP_FNC) {
        return tp_lua_hash(&v.fnc.val,2*sizeof(void*));
    } else if (type == TP_PARAMS) {
        return tp_lua_hash(&v.list.val,2*sizeof(void*));
    } else if (type == TP_DATA) {
        return tp_lua_hash(&v.data.val,sizeof(void*));
    }
    tp_raise(0,"tp_hash(%s)",STR(v));
}

void _tp_dict_hash_set(TP,_tp_dict *self, int hash, tp_obj k, tp_obj v) {
    int i,idx = hash&self->mask;
    for (i=idx; i<idx+self->alloc; i++) {
        int n = i&self->mask;
        if (self->items[n].used > 0) { continue; }
        if (self->items[n].used == 0) { self->used += 1; }
        self->items[n] = (tp_item){1,hash,k,v};
        self->len += 1;
        return;
    }
    tp_raise(,"_tp_dict_hash_set(%d,%d,%s,%s)",self,hash,STR(k),STR(v));
}

void _tp_dict_tp_realloc(TP,_tp_dict *self,int len) {
    len = _tp_max(8,len);
    tp_item *items = self->items;
    int i,alloc = self->alloc;

    self->items = tp_malloc(len*sizeof(tp_item));
    self->alloc = len; self->mask = len-1;
    self->len = 0; self->used = 0;

    for (i=0; i<alloc; i++) {
        if (items[i].used != 1) { continue; }
        _tp_dict_hash_set(vm,self,items[i].hash,items[i].key,items[i].val);
    }
    tp_free(items);
}

int _tp_dict_hash_find(TP,_tp_dict *self, int hash, tp_obj k) {
    int i,idx = hash&self->mask;
    for (i=idx; i<idx+self->alloc; i++) {
        int n = i&self->mask;
        if (self->items[n].used == 0) { break; }
        if (self->items[n].used < 0) { continue; }
        if (self->items[n].hash != hash) { continue; }
        if (tp_cmp(vm,self->items[n].key,k).number.val != 0) { continue; }
        return n;
    }
    return -1;
}
int _tp_dict_find(TP,_tp_dict *self,tp_obj k) {
    return _tp_dict_hash_find(vm,self,tp_hash(vm,k),k);
}

void _tp_dict_setx(TP,_tp_dict *self,tp_obj k, tp_obj v) {
    int hash = tp_hash(vm,k); int n = _tp_dict_hash_find(vm,self,hash,k);
    if (n == -1) {
        if (self->len >= (self->alloc/2)) {
            _tp_dict_tp_realloc(vm,self,self->alloc*2);
        } else if (self->used >= (self->alloc*3/4)) {
            _tp_dict_tp_realloc(vm,self,self->alloc);
        }
        _tp_dict_hash_set(vm,self,hash,k,v);
    } else {
        self->items[n].val = v;
    }
}

void _tp_dict_set(TP,_tp_dict *self,tp_obj k, tp_obj v) {
    _tp_dict_setx(vm,self,k,v);
    tp_grey(vm,k); tp_grey(vm,v);
}

tp_obj _tp_dict_get(TP,_tp_dict *self,tp_obj k, char *error) {
    int n = _tp_dict_find(vm,self,k);
    if (n < 0) {
        tp_raise(None,"%s: KeyError: %s\n",error,STR(k));
    }
    return self->items[n].val;
}

void _tp_dict_del(TP,_tp_dict *self,tp_obj k, char *error) {
    int n = _tp_dict_find(vm,self,k);
    if (n < 0) { tp_raise(,"%s: KeyError: %s\n",error,STR(k)); }
    self->items[n].used = -1;
    self->len -= 1;
}

_tp_dict *_tp_dict_new(void) {
    _tp_dict *self = tp_malloc(sizeof(_tp_dict));
    return self;
}
tp_obj _tp_dict_copy(TP,tp_obj rr) {
    _tp_dict *o = rr.dict.val;
    _tp_dict *r = _tp_dict_new(); *r = *o;
    r->items = tp_malloc(sizeof(tp_item)*o->alloc);
    memcpy(r->items,o->items,sizeof(tp_item)*o->alloc);
    
    rr.type = TP_DICT;
    rr.dict.val = r;
    tp_track(vm,rr);
//     tp_follow(vm,rr);
    return rr;
}

int _tp_dict_next(TP,_tp_dict *self) {
    if (!self->len) { tp_raise(0,"_tp_dict_next(...)",0); }
    while (1) {
        self->cur = ((self->cur + 1) & self->mask);
        if (self->items[self->cur].used > 0) {
            return self->cur;
        }
    }
}

tp_obj tp_merge(tp_obj params) {
    TP_PBEGIN(params);
    tp_obj self = TP_POBJ("self");
    tp_obj v = TP_POBJ("val");
    TP_PEND;
    int i;
    for (i=0; i<v.dict.val->len; i++) {
        int n = _tp_dict_next(vm,v.dict.val);
        _tp_dict_set(vm,self.dict.val,
            v.dict.val->items[n].key,v.dict.val->items[n].val);
    }
    return None;
}

tp_obj tp_dict_new(void) {
    tp_obj r;
    r.type = TP_DICT;
    r.dict.val = _tp_dict_new();
    return r;
}

tp_obj tp_dict(TP,int len, tp_obj* argv) {
    int i;
    tp_obj r = tp_dict_new();
    for (i=0; i<len; i+=2) {
        tp_obj k = argv[i];
        tp_obj v = argv[i+1];
        tp_set(vm,r,k,v);
    }
//     r.list.hash = lua_hash(&r.dict.val,sizeof(void*));
    tp_track(vm,r);
    return r;
}

